﻿/// <reference name="MicrosoftAjax.js"/>

Type.registerNamespace("JData");
JData.Expression = function() {
};

JData.Expression.prototype = {
    calculate: function() {
        throw Error.notImplemented();
    }
};

JData.Expression.TokenId = function() {
};
JData.Expression.TokenId.prototype = {
    OpenParen: 0,
    CloseParen: 1,
    Asterisk: 2,
    Plus: 3,
    Minus: 4,
    Slash: 5,
    Identifier: 6,
    IntegerLiteral: 7,
    End: 8
};


JData.Expression.TokenId.registerEnum('JData.Expression.TokenId', true);

JData.Expression.Parser = function(text) {
    this.token = {};
    this.text = text;
    this.textLen = text.length;
    this._setTextPos(0);
};

JData.Expression.Parser.prototype = {
    _setTextPos: function(pos) {
        this.textPos = pos;
        this.ch = this.textPos < this.textLen ? this.text[this.textPos] : '\0';
    },
    _isLetter: function(s) {
        var patrn = /[A-Za-z]/;

        if (!patrn.exec(s))
            return false;
        return true;
    },
    _isDigit: function(s) {
        var patrn = /[0-9]/;

        if (!patrn.exec(s))
            return false;
        return true;
    },
    _nextChar: function() {
        if (this.textPos < this.textLen)
            this.textPos = this.textPos + 1;

        this.ch = this.textPos < this.textLen ? this.text[this.textPos] : '\0';
    },
    _nextToken: function() {
        var tokenPos = this.textPos;
        switch (this.ch) {
            case '+':
                this._nextChar();
                this.t = JData.Expression.TokenId.Plus;
                break;
            case '-':
                this._nextChar();
                this.t = JData.Expression.TokenId.Minus;
                break;
            case '*':
                this._nextChar();
                this.t = JData.Expression.TokenId.Asterisk;
                break;
            case '/':
                break;
            case '(':
                this._nextChar();
                this.t = JData.Expression.TokenId.OpenParen;
                break;
            case ')':
                this._nextChar();
                this.t = JData.Expression.TokenId.CloseParen;
                break;
            default:
                if (this._isLetter(this.ch)) {
                    do {
                        this._nextChar();
                    } while (this._isLetter(this.ch) || this._isDigit(this.ch));
                    this.t = JData.Expression.TokenId.Identifier;
                    break;
                }
                if (this._isDigit(this.ch)) {
                    this.t = JData.Expression.TokenId.IntegerLiteral;
                    do {
                        this._nextChar();
                    } while (this._isDigit(this.ch));
                    break;
                }
                if (this.ch == '.') {
                    this.t = JData.Expression.TokenId.RealLiteral;
                    this._nextChar();
                    do {
                        this._nextChar();
                    } while (this._isDigit(this.ch));
                    break;
                }
                if (this.textPos == this.textLen) {
                    this.t = JData.Expression.TokenId.End;
                    break;
                }
                throw Error.create('Parse Error');
        }
        this.tokenText = this.text.substr(tokenPos, this.textPos - tokenPos);
        this.token.id = this.t;
        this.token.text = this.text.substr(tokenPos, this.textPos - tokenPos);
        this.token.pos = tokenPos;
    },
    _parsePrimaryStart: function() {
        switch (token.id) {
            case JData.Expression.TokenId.Identifier:
                return this._parseIdentifier();
            case JData.Expression.TokenId.StringLiteral:
                return this._parseStringLiteral();
            case JData.Expression.TokenId.IntegerLiteral:
                return this._parseIntegerLiteral();
            case JData.Expression.TokenId.RealLiteral:
                return this._parseRealLiteral();
            case JData.Expression.TokenId.OpenParen:
                return this._parseParenExpression();
            default:
                throw Error.create('ExpressionExpected'); //ParseError(Res.ExpressionExpected);
        }
    },
    _parsePrimary: function() {
        var expr = this._parsePrimaryStart();
        //        while (true) {
        //            if (token.id == JData.Expression.TokenId.Dot) {
        //                NextToken();
        //                expr = this.parseMemberAccess(null, expr);
        //            }
        //            else if (token.id == JData.Expression.TokenId.OpenBracket) {
        //                expr = this._parseElementAccess(expr);
        //            }
        //            else {
        //                break;
        //            }
        //        }
        return expr;
    },

    // -, !, not unary operators
    _parseUnary: function() {
        if (this.token.id == JData.Expression.TokenId.Minus) {
            var op = this.token;
            this._nextToken();
            if (op.id == JData.Expression.TokenId.Minus && (this.token.id == JData.Expression.TokenId.IntegerLiteral ||
                                                            this.token.id == JData.Expression.TokenId.RealLiteral)) {
                this.token.text = '-' + this.token.text;
                this.token.pos = op.pos;
                return this._parsePrimary();
            }
            var expr = this._parseUnary();
            if (op.id == JData.Expression.TokenId.Minus) {
                expr = Expression.Negate(expr);
            }
        }
    },

    _parseMultiplicative: function() {
        //     Expression left = ParseUnary();
        //            while (token.id == TokenId.Asterisk || token.id == TokenId.Slash ||
        //                token.id == TokenId.Percent || TokenIdentifierIs("mod"))
        //            {
        //                Token op = token;
        //                NextToken();
        //                Expression right = ParseUnary();
        //                CheckAndPromoteOperands(typeof(IArithmeticSignatures), op.text, ref left, ref right, op.pos);
        //                switch (op.id)
        //                {
        //                    case TokenId.Asterisk:
        //                        left = Expression.Multiply(left, right);
        //                        break;
        //                    case TokenId.Slash:
        //                        left = Expression.Divide(left, right);
        //                        break;
        //                    case TokenId.Percent:
        //                    case TokenId.Identifier:
        //                        left = Expression.Modulo(left, right);
        //                        break;
        //                }
        //            }
        //            return left;
    },

    // +, -, & operators
    _parseAdditive: function() {
    },

    parse: function() {
        var left = this._parsePrimary();
    }
};
JData.Expression.Parser.registerClass('JData.Expression.Parser');



JData.Expression.parse = function(text) {
    /// <param name="text", type="String"/>
    var i = 0;
    while (i < text.length) {

        i++;
    };

    isLetter = function(s) {
        var patrn = /[A - Za - z]/;

        if (!patrn.exec(s))
            return false;
        return true;
    }
    isDigit = function(s) {
        var patrn = /[0-9]/;

        if (!patrn.exec(s))
            return false;
        return true;
    }

    var ch, textPos, textLen, t, tokenText;

    nextChar = function() {
        if (textPos < textLen)
            textPost++;
        textPos < textLen ? text[textPos] : '\0';
    }


    nextToken = function() {
        var tokenPos = textPos;
        switch (ch) {
            case '+':
                break;
            case '-':
                break;
            case '*':
                break;
            case '/':
                break;
            case '(':
                nextChar();
                t = JData.Expression.TokenId.OpenParen;
                break;
            case ')':
                nextChar();
                t = Data.Expression.TokenId.CloseParen;
                break;
            default:
                if (isLetter(ch)) {
                    do {
                        nextChar();
                    } while (isLetter(ch) || isDigit(ch));
                    t = JData.Expression.TokenId.Identifier;
                    break;
                }
                if (isDigit(ch)) {
                    t = JData.Expression.TokenId.IntegerLiteral;
                    do {
                        nextChar();
                    } while (isDigit(ch));
                    break;
                }
                if (ch == '.') {
                    t = JData.Expression.TokenId.RealLiteral;
                    nextChar();
                    do {
                        nextChar();
                    } while (isDigit(ch));
                }
                throw Error.create('Parse Error');
        }
        tokenText = text.substr(tokenPos, textPos - tokenPos);
    }


},

JData.Expression.registerClass('JData.Expression');

JData.MemberExpression = function() {
};

JData.MemberExpression.prototype = {
    get_memberName: function() {
        return this._memberName;
    },
    set_memberName: function(value) {
        this._memberName = value;
    },

    get_dataItem: function() {
        return this._dataItem;
    },
    set_dataItem: function(value) {
        this._dataItem = value;
    },

    calculate: function() {
        return this.get_dataItem()[this.get_memberName()];
    }
};

JData.MemberExpression.registerClass('JData.MemberExpression', JData.Expression);



JData.ConstantExpression = function() {
};

JData.ConstantExpression.prototype = {
    get_value: function() {
        return this._value;
    },
    set_value: function(value) {
        this._value = value;
    },

    calculate: function() {
        return this.get_value();
    }
};

JData.ConstantExpression.registerClass('JData.ConstantExpression', JData.Expression);


JData.BinaryExpression = function() {
};

JData.BinaryExpression.prototype = {
    get_leftExpression: function() {
        return this._leftExpression;
    },
    set_leftExpression: function(value) {
        this._leftExpression = value;
    },

    get_rightExpression: function() {
        return this._rightExpression;
    },
    set_rightExpression: function(value) {
        this._rightExpression = value;
    },

    get_operator: function() {
        return this._operator;
    },
    set__operator: function(value) {
        this._operator = value;
    },

    calculate: function() {
        switch (this.get_operator()) {
            case '+':
                return this.get_leftExpression().calculate() + this.get_rightExpression().calculate();
            case '-':
                return this.get_leftExpression().calculate() - this.get_rightExpression().calculate();
            case '*':
                return this.get_leftExpression().calculate() * this.get_rightExpression().calculate();
            case '/':
                return this.get_leftExpression().calculate() / this.get_rightExpression().calculate();
            default:
                throw Error.notImplemented();
        }
    }
};

JData.BinaryExpression.registerClass('JData.BinaryExpression', JData.Expression);