﻿/// <reference name="MicrosoftAjax.js"/>
/// <reference path="Enum.js"/>
/// <reference path="Style.js"/>

Type.registerNamespace("JData");

(function() {
    JData.GroupExpandCell = function(element) {
        if (element == null)
            throw Error.argumentNull('element');

        JData.TreeColumnCell.initializeBase(this, new Array(element));

        this._expanded = false;
        this._childRows = new Array();
    };

    JData.GroupExpandCell.prototype = {

        get_expanded: function() {
            return this._expanded;
        },

        get_controlValue: function() {
            if (this._createControl == null)
                return null;

            var value = $(this._createControl).attr('value');
            var type = this.get_containingField().get_valueType();

            if (type == Boolean) {
                value = Boolean.parse(value);
            }
            else if (type == Number) {
                value = Number.parseLocale(value);
            }
            else if (type == Date) {
                value = Date.parse(value);
            }

            return value;
        },
        set_controlValue: function(value) {
            if (this._createControl == null)
                return;

            var text = value == null ? this.get_containingField().get_nullText() : value;
            $(this._createControl).attr('value', text);
        },

        get_contentContainer: function() {
            return this.contentContainer;
        },

        //======================== EVENTS =============================

        add_nodeExpanded: function(handler) {
            this.get_events().addHandler('nodeExpanded', handler);
        },
        remove_nodeExpanded: function(handler) {
            this.get_events().removeHandler('nodeExpanded', handler);
        },

        //=============================================================

        expand: function() {
            var gridView = this.get_parent().get_parent();
            var dataItem = this.get_parent().get_dataItem();
            var index = Array.indexOf(gridView.get_rows(), this.get_parent());
            var cell = this;
            if (this._expanded == false) {
                var children = gridView.get_dataSource().getChildren(dataItem);
                if (children != null) {
                    for (var i = 0; i < children.length; i++) {
                        var child = children[i];
                        var properties = {
                            dataItem: child,
                            parent: gridView,
                            rowType: JData.DataControlRowType.DataRow
                        };
                        var row = gridView._CreateRow(index + i + 1, properties);
                        Array.add(this._childRows, row);

                        var style;
                        if (i / 2 != Math.round(i / 2))
                            style = gridView.get_alternatingRowStyle();
                        else
                            style = gridView.get_rowStyle();
                        row.applyStyle(style);
                    }
                }
            }
            else {
                while (this._childRows.length > 0) {
                    var childRow = this._childRows.pop();
                    gridView._RemoveRow(childRow);
                }
            }

            Sys.Debug.assert(this._expanded != null);
            this._expanded = !this._expanded;

            var h = this.get_events().getHandler('nodeExpanded');
            if (h) h(this, {});
        },

        _getChildren: function(dataItem) {
            var gridView = this.get_parent().get_parent(); //this.get_containingField().gridView;
            var dataSource = gridView.get_dataSource();
            if (dataSource.getChildren == null)
                throw Error.argumentNull('dataSource.getChildren');

            return dataSource.getChildren(dataItem);
        },

        removeChildRows: function() {
            while (this._childRows.length > 0) {
                var childRow = this._childRows.pop();
                this.get_parent().get_parent()._RemoveRow(childRow);
            }
        },

        deleteChildRows: function() {
            while (this._childRows.length > 0) {
                var childRow = this._childRows.pop();
                gridView._HandleDelete(childRow);
            }
        },

        initialize: function() {
            var row = this.get_parent();
            var children = this._getChildren(row.get_dataItem());
            var gridView = this.get_parent().get_parent();
            var cell = this;
            var col = this.get_containingField();

            this.contentContainer = document.createElement('span');
            $(this.contentContainer).css('padding-top', '4px');
            $(this.contentContainer).css('float', 'left');

            this.set_value();
            if (children != null && children.length > 0) {
                var button = document.createElement('span');
                $(button).addClass('ui-icon ui-icon-plus');
                button.style.marginLeft = '0px';
                button.style.cssFloat = 'left'; //for firefox and chorme
                $(button).css('float', 'left'); //for IE and opera
                $(button).addClass('ui-icon');
                var url = document.createElement('a');
                url.href = 'javascript:';
                url.appendChild(button);
                this.get_element().appendChild(url);

                $addHandler(url, 'click', function(args) {
                    cell.expand();
                });

                this.add_nodeExpanded(function(sender, args) {
                    if (sender.get_expanded() == true) {
                        //button.className = ('ui-icon ui-icon-minus');
                        $(button).removeClass('ui-icon-plus');
                        $(button).addClass('ui-icon-minus');
                    }
                    else {
                        //button.className = ('ui-icon ui-icon-plus');
                        $(button).removeClass('ui-icon-minus');
                        $(button).addClass('ui-icon-plus');
                    }
                });
            }
            else {
                this.contentContainer.style.marginLeft = (1 * 10 + 8) + 'px';
            }
            this.get_element().appendChild(this.contentContainer);

            row.add_valueChanged(function(sender, args) {
                Sys.Debug.assert(Array.isInstanceOfType(args.fields));
                if (Array.contains(args.fields, col.get_dataField())) {
                    cell.set_value();
                }
            });
        }
    };
    JData.GroupExpandCell.registerClass('JData.GroupExpandCell', JData.EditableCell);
})();


JData.GroupRow = function(element) {
    JData.GroupRow.initializeBase(this, new Array(element));
};

JData.GroupRow.prototype = {
    get_column: function() {
        if (this._column == null) throw Error.argumentNull('column');

        return this._column;
    },
    set_column: function(value) {
        this._column = value;
    },

    _createCells: function() {
        var gridView = this.get_parent();
        var columns = gridView.getVisibleColumns();
        var colSpan = 1;
        for (var j = 0; j < columns.length; j++) {
            var col = columns[j];
            if (col == this.get_column()) {
                var cellElement = this.get_element().insertCell(this.get_element().cells.length);
                var properties = { parent: this, containingField: col, parent: this };
                this.cell = $create(JData.GroupExpandCell, properties, null, null, cellElement); //col._CreateCell(this);
                Array.add(this.get_cells(), this.cell);
            }
            else if (JData.CommandField.isInstanceOfType(col)) {
                var cell = col._CreateCell(this);
                Array.add(this.get_cells(), cell);
            }
            else {
                colSpan++;
            }
        }
        $(this.cell.get_element()).attr('colSpan', colSpan);
    }
};

JData.GroupRow.registerClass('JData.GroupRow', JData.GridViewRow); // JData.TableRow);

JData.GroupRow.applyTo = function(gridView, column, style, cascadeDelete) {
    /// <param name="gridView" type="JData.GridView"/>
    /// <param name="column" type="JData.BoundField"/>
    /// <param name="style" type="JData.Style"/>
    if (gridView == null) throw Error.argumentNull('gridView');
    if (column == null) throw Error.argumentNull('column');

    var orgGetRowType = gridView._GetRowType;
    var newGetRowType = function(index, properties) {
        if (properties.rowType == JData.DataControlRowType.DataRow && properties.dataItem.Key != null) {
            properties.column = column;
            return JData.GroupRow;
        }
        return orgGetRowType.apply(gridView, new Array(index, properties));
    }
    gridView._GetRowType = newGetRowType;

    if (style == null) {
        style = gridView.get_rowStyle();
    }
    var orgGetRowStyle = gridView._GetRowStyle;
    var newGetRowStyle = function(row) {
        if (row.get_rowType() == JData.DataControlRowType.DataRow)
            return style;
        var index = row.get_rowIndex();
        return orgGetRowStyle.apply(gridView, new Array(row));
    }
    gridView._GetRowStyle = newGetRowStyle;


    var parent;
    gridView.add_rowUpdating(function(sender, args) {
        var dataSource = gridView.get_dataSource();
        var dataItem = args.row.get_dataItem();
        if (dataSource.getParent == null) throw Error.notImplemented('dataSource.getParent(dataItem) method is not implemented.');

        parent = dataSource.getParent(dataItem);
    });

    gridView.add_rowUpdated(function(sender, args) {
        var dataSource = gridView.get_dataSource();
        var dataItem = args.row.get_dataItem();
        if (dataSource.getParent == null) throw Error.notImplemented('dataSource.getParent(dataItem) method is not implemented.');

        var current = dataSource.getParent(dataItem);
        if (parent != current) {
            gridView._RemoveRow(args.row);

            var gridRow;
            var gridRows = gridView.get_rows();
            for (var i = 0; i < gridRows.length; i++) {
                if (gridRows[i].get_dataItem() == current) {
                    gridRow = gridRows[i];
                    break;
                }
            }
            if (gridRow.cell.get_expanded()) {
                gridRow.cell.expand();
                gridRow.cell.expand();
            }
        }
    });

    gridView.add_rowDeleting(function(sender, args) {
        var gridRow = args.row;
        if (JData.GroupRow.isInstanceOfType(gridRow) == false)
            return;

        if (cascadeDelete == true)
            gridRow.cell.deleteChildRows();
        else
            gridRow.cell.removeChildRows();
    });
};

