﻿/// <reference name="MicrosoftAjax.js"/>

Type.registerNamespace("JData");

//============================= JQueryUIStyle =============================

JData.JQueryUIStyle = function(view, type, options) {
    /// <param name="view">
    /// The view can be a GridView or DetailView or FormView or JSON object.
    /// </param>
    /// <param name="type" type="Type" mayBeNull="true">
    /// The type can be JData.GridView or JData.DetailsView or JData.FormView.
    /// </param>
    /// <param name="options" type="JSON" mayBeNull="true"/>

    if (type == null)
        type = Object.getType(view, options);

    if (type == JData.GridView)
        JData.JQueryUIStyle.gridView(view, options);

    if (type == JData.DetailsView)
        JData.JQueryUIStyle.detailsView(view, options);

    if (type == JData.FormView)
        JData.JQueryUIStyle.formView(view, options);
};

JData.JQueryUIStyle.gridView = function(gridViewOrProperties, options) {

    if (options == null)
        options = {};

    if (options.autoUpdateStyle == null)
        options.autoUpdateStyle = true;

    var isGridView = JData.GridView.isInstanceOfType(gridViewOrProperties);
    var gridView = gridViewOrProperties;
    var properties = gridViewOrProperties;

    if (isGridView) {
        gridView.set_gridLines(JData.GridLines.Both);
        gridView.set_cssClass('ui-widget ui-widget-content');
    }
    else {
        properties.gridLine = JData.GridLines.Both;
        properties.cssClass = 'ui-widget ui-widget-content';
    }


    //Set the caption style
    var style;
    if (isGridView)
        style = gridView.get_captionStyle();
    else {
        style = new JData.TableItemStyle()
        properties.captionStyle = style;
    }

    style.set_cssClass('ui-widget-header');
    style.set_height('26px');


    //Set the header style
    if (isGridView)
        style = gridView.get_headerStyle();
    else {
        style = new JData.TableItemStyle();
        properties.headerStyle = style;
    }
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    style.set_horizontalAlign('center');

    //Set the empty row style
    if (isGridView) {
        style = gridView.get_emptyDataRowStyle();
    }
    else {
        style = new JData.TableItemStyle();
        properties.emptyDataRowStyle = style;
    }
    style.set_height('200px');
    style.set_horizontalAlign('center');

    //Set the row style
    if (isGridView)
        style = gridView.get_rowStyle();
    else {
        style = new JData.TableItemStyle();
        properties.rowStyle = style;
    }
    style.set_height('26px');

    //Set the alternating row style
    if (isGridView)
        style = gridView.get_alternatingRowStyle();
    else {
        style = new JData.TableItemStyle();
        properties.alternatingRowStyle = style;
    }
    style.set_backColor("#ffffff");
    style.set_height('26px');

    //Set the selected row style.
    if (isGridView)
        style = gridView.get_selectedRowStyle();
    else {
        style = new JData.TableItemStyle();
        properties.selectedRowStyle = style;
    }
    style.set_cssClass("ui-state-highlight");
    style.set_height('26px');

    //Set the edited row style.
    if (isGridView)
        style = gridView.get_editRowStyle();
    else {
        style = new JData.TableItemStyle();
        properties.editRowStyle = style;
    }
    style.set_cssClass("ui-state-highlight");
    style.set_height('26px');
    
    //Set the pager style
    if (isGridView)
        style = gridView.get_pagerStyle();
    else {
        style = new JData.Style();
        properties.pagerStyle = style;
    }
    style.set_cssClass('ui-state-hover');
    style.set_height('24px');

    //Set the footer style
    if (isGridView)
        style = gridView.get_footerStyle();
    else {
        style = new JData.TableItemStyle();
        properties.footerStyle = style;
    }
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    style.set_horizontalAlign('center');


    if (options.autoUpdateStyle) {
        var rowIndex;
        gridView.add_rowRemoving(function(sender, args) {
            if (args.row.get_rowType() == JData.DataControlRowType.DataRow)
                rowIndex = args.row.get_rowIndex();
        });
        gridView.add_rowRemoved(function(sender, args) {
            if (args.row.get_rowType() == JData.DataControlRowType.DataRow) {
                gridView.updateRowsStyle(rowIndex);
            }
        });
        gridView.add_rowCreated(function(sender, args) {
            if (args.row.get_rowType() == JData.DataControlRowType.DataRow) {
                rowIndex = args.row.get_rowIndex();
                gridView.updateRowsStyle(rowIndex);
            }
        });
    }
}

JData.JQueryUIStyle.detailsView = function(gridViewOrProperties) {
    var isGridView = JData.DetailsView.isInstanceOfType(gridViewOrProperties);
    var gridView = gridViewOrProperties;
    var properties = gridViewOrProperties;

    if (isGridView) {
        gridView.set_gridLines(JData.GridLines.Both);
        gridView.set_cssClass('ui-widget ui-widget-content');
    }
    else {
        properties.gridLine = JData.GridLines.Both;
        properties.cssClass = 'ui-widget ui-widget-content';
    }

    //Set the caption style
    var style = new JData.TableItemStyle();
    style.set_cssClass('ui-widget-header');
    style.set_height('26px');
    if (isGridView)
        gridView.set_captionStyle(style);
    else
        properties.captionStyle = style;

    //Set the header style
    style = new JData.TableItemStyle();
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    style.set_horizontalAlign('center');
    if (isGridView)
        gridView.set_headerStyle(style);
    else
        properties.headerStyle = style;

    //Set the empty row style
    style = new JData.TableItemStyle();
    style.set_height('200px');
    style.set_horizontalAlign('center');
    if (isGridView)
        gridView.set_emptyDataRowStyle(style);
    else
        properties.emptyDataRowStyle = style;

    //Set the row style
    style = new JData.TableItemStyle();
    style.set_height('26px');
    if (isGridView)
        gridView.set_rowStyle(style);
    else
        properties.rowStyle = style;

    //Set the alternating row style
    style = new JData.TableItemStyle();
    style.set_backColor("#ffffff");
    style.set_height('26px');
    if (isGridView)
        gridView.set_alternatingRowStyle(style);
    else
        properties.alternatingRowStyle = style;

    style = new JData.TableItemStyle();
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    if (isGridView)
        gridView.set_commandRowStyle(style);
    else
        properties.commandRowStyle = style;

    //Set the pager style
    style = new JData.Style();
    style.set_cssClass('ui-state-hover');
    style.set_height('24px');
    if (isGridView)
        gridView.set_pagerStyle(style);
    else
        properties.pagerStyle = style;

    //Set the footer style
    style = new JData.TableItemStyle();
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    style.set_horizontalAlign('center');
    if (isGridView)
        gridView.set_footerStyle(style);
    else
        properties.footerStyle = style;
}

JData.JQueryUIStyle.formView = function(formViewOrProperties) {
    var gridViewOrProperties = formViewOrProperties;
    var isGridView = JData.FormView.isInstanceOfType(gridViewOrProperties);
    var gridView = gridViewOrProperties;
    var properties = gridViewOrProperties;

    if (isGridView) {
        gridView.set_gridLines(JData.GridLines.Both);
        gridView.set_cssClass('ui-widget ui-widget-content');
    }
    else {
        properties.gridLine = JData.GridLines.Both;
        properties.cssClass = 'ui-widget ui-widget-content';
    }


    //Set the caption style
    var style = new JData.TableItemStyle();
    style.set_cssClass('ui-widget-header');
    style.set_height('26px');
    if (isGridView)
        gridView.set_captionStyle(style);
    else
        properties.captionStyle = style;

    //Set the header style
    style = new JData.TableItemStyle();
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    style.set_horizontalAlign('center');
    if (isGridView)
        gridView.set_headerStyle(style);
    else
        properties.headerStyle = style;

    //Set the empty row style
    style = new JData.TableItemStyle();
    style.set_height('200px');
    style.set_horizontalAlign('center');
    if (isGridView)
        gridView.set_emptyDataRowStyle(style);
    else
        properties.emptyDataRowStyle = style;

    //Set the row style
    style = new JData.TableItemStyle();
    style.set_height('26px');
    if (isGridView)
        gridView.set_rowStyle(style);
    else
        properties.rowStyle = style;

    //Set the alternating row style
    style = new JData.TableItemStyle();
    style.set_backColor("#ffffff");
    style.set_height('26px');
    if (isGridView)
        gridView.set_alternatingRowStyle(style);
    else
        properties.alternatingRowStyle = style;

    style = new JData.TableItemStyle();
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    if (isGridView)
        gridView.set_commandRowStyle(style);
    else
        properties.commandRowStyle = style;

    //Set the pager style
    style = new JData.Style();
    style.set_cssClass('ui-state-hover');
    style.set_height('24px');
    if (isGridView)
        gridView.set_pagerStyle(style);
    else
        properties.pagerStyle = style;

    //Set the footer style
    style = new JData.TableItemStyle();
    style.set_height('26px');
    style.set_cssClass('ui-state-default');
    style.set_horizontalAlign('center');
    if (isGridView)
        gridView.set_footerStyle(style);
    else
        properties.footerStyle = style;
} 