﻿/// <reference name="MicrosoftAjax.js"/>
/// <reference path="Enum.js"/>
/// <reference path="../Script/jquery-1.4.1-vsdoc.js"/>

Type.registerNamespace("JData");

(function() {
    JData.TreeColumnCell = function(element) {
        if (element == null)
            throw Error.argumentNull('element');

        JData.TreeColumnCell.initializeBase(this, new Array(element));
        this._childCells = null;
        this.set_expandDepth(1);
        this._expanded = false;
    };

    JData.TreeColumnCell.prototype = {
        get_expandDepth: function() {
            return this._expandDepth;
        },
        set_expandDepth: function(value) {
            this._expandDepth = value;
        },

        get_parentCell: function() {
            return this._parentCell;
        },
        set_parentCell: function(value) {
            this._parentCell = value;
        },

        get_childCells: function() {
            return this._childCells;
        },

        get_childRows: function() {
            var cells = this.get_childCells();
            if (cells == null)
                return null;

            var result = new Array();
            for (var i = 0; i < cells.length; i++) {
                var row = cells[i].get_parent();
                Array.add(result, row);
            }
            return result;
            //return this._childRows;
        },

        _addChild: function(child) {
            Array.insert(this._childCells, 0, child);
            child.set_parentCell(this);
        },

        get_depth: function() {
            var gridRow = this.get_parent();
            var gridView = gridRow.get_parent();
            var dataSource = gridView.get_dataSource();

            var depth = 0;
            var parent = gridRow.get_dataItem();

            while (parent != dataSource.get_root()) {
                depth = depth + 1;
                parent = dataSource.getParent(parent);
            }
            if (dataSource.get_displayRoot() == false)
                return depth - 1;

            return depth;
        },

        get_expanded: function() {
            return this._expanded;
        },

        createControl: function() {
            if (this._createControl == null) {
                this._createControl = document.createElement('input');
            }
            return this._createControl;
        },

        //======================== EVENTS =============================

        add_treeNodeExpanded: function(handler) {
            this.get_events().addHandler('treeNodeExpanded', handler);
        },
        remove_treeNodeExpanded: function(handler) {
            this.get_events().removeHandler('treeNodeExpanded', handler);
        },

        //=============================================================

        _refresh: function() {
            if (this.get_childCells() == null)
                this._createChildren();
            else {
                for (var i = 0; i < this.get_childCells().length; i++) {
                    var cell = this.get_childCells()[i];
                    cell._refresh();
                    if (this.get_expanded() == false) {
                        Sys.UI.DomElement.setVisible(cell.get_parent().get_element(), false);
                    }
                    else {
                        Sys.UI.DomElement.setVisible(cell.get_parent().get_element(), true);
                    }
                }
            }

        },

        expand: function() {
            var gridView = this.get_parent().get_parent();

            this._expanded = !this._expanded;
            this._setButtonStyle();

            if (this._expanded == false) {
                var stack = new Array();
                for (var i = 0; i < this.get_childCells().length; i++) {
                    stack.push(this.get_childCells()[i]);
                }
                while (stack.length > 0) {
                    var item = stack.pop();
                    Sys.UI.DomElement.setVisible(item.get_parent().get_element(), false);
                    if (item.get_childCells() != null) {
                        for (var i = 0; i < item.get_childCells().length; i++) {
                            stack.push(item.get_childCells()[i]);
                        }
                    }
                }
            }
            else {
                this._refresh();
            }

            //========================================================

            var h = this.get_events().getHandler('treeNodeExpanded');
            if (h) h(this, {});
        },

        expandAll: function() {
            var stack = new Array();
            stack.push(this);
            while (stack.length > 0) {
                var cell = stack.pop();
                cell.expand();
                for (var i = 0; i < cell.get_childRows().length; i++) {
                    stack.push(cell.get_childRows()[i].get_cells()[cell.get_containingField().get_index()]);
                }
            }
        },

        get_row: function() {
            return this.get_parent();
        },

        get_gridView: function() {
            return this.get_parent().get_parent();
        },

        _getAllDisplayChildrenCount: function(cell) {
            var childCells = cell.get_childCells();
            if (!cell.get_expanded() || childCells == null) {
                return 0;
            }

            var count = childCells.length;
            for (var i = 0; i < childCells.length; i++) {
                count = count + this._getAllDisplayChildrenCount(childCells[i]);
            }
            return count;
        },

        _createChild: function(index, dataItem) {

            var childCells = this.get_childCells();
            if (childCells == null) {
                this._createChildren();
                return;
            }

            var rowIndex = this.get_parent().get_rowIndex();
            for (var i = 0; i < childCells.length && i < index; i++) {
                rowIndex = rowIndex + 1;
                rowIndex = rowIndex + this._getAllDisplayChildrenCount(childCells[i]);
            }
            rowIndex = rowIndex + 1;
            var row = this.get_gridView()._CreateDataRow(rowIndex, dataItem);
            Array.add(this.get_childCells(), row.get_cells()[this.get_containingField().get_index()]);
            row.get_cells()[this.get_containingField().get_index()].set_parentCell(this);
            if (this.get_visible() == false || this.get_expanded() == false) {
                Sys.UI.DomElement.setVisible(row.get_element(), false);
            }
            else {
                Sys.UI.DomElement.setVisible(row.get_element(), true);
            }
            this._showExpandButton();
        },



        _createChildren: function() {
            Sys.Debug.assert(this._childRows == null);
            this._childRows = new Array();
            this._childCells = new Array();
            var dataItem = this.get_row().get_dataItem();
            var children = this.get_gridView().get_dataSource().getChildren(dataItem);
            var gridView = this.get_parent().get_parent();
            if (children != null && children.length > 0) {
                for (var i = 0; i < children.length; i++) {
                    this._createChild(i, children[i]);
                }
            }
        },

        _removeChild: function(cell) {
            Sys.Debug.assert(cell != null);
            this.get_gridView()._RemoveRow(cell.get_parent());
            Array.remove(this.get_childCells(), cell);
            cell.set_parentCell(null);
            if (this.get_childCells().length == 0)
                this._hideExpandButton();
        },

        _removeChildren: function() {
            var gridView = this.get_parent().get_parent();
            var colIndex = this.get_containingField().get_index();

            if (this.get_childCells() == null || this.get_childCells().length < 1)
                return;

            while (this.get_childCells().length > 0) {
                var child = this.get_childCells()[0];
                child._removeChildren();
                this._removeChild(child);
            }
            this._childRows = null;
            this._childCells = null;
        },

        _setButtonStyle: function() {
            this._expandButton.style.cssText = this._expandButton.style.cssText + ';float:left';
            if (this.get_expanded() == false) {
                Sys.UI.DomElement.removeCssClass(this._expandButton, 'ui-icon-minus');
                Sys.UI.DomElement.addCssClass(this._expandButton, 'ui-icon-plus');
            }
            else {
                Sys.UI.DomElement.removeCssClass(this._expandButton, 'ui-icon-plus');
                Sys.UI.DomElement.addCssClass(this._expandButton, 'ui-icon-minus');
            }
        },

        _getChildren: function(dataItem) {
            var gridView = this.get_parent().get_parent();
            var dataSource = gridView.get_dataSource();
            if (dataSource.getChildren != null) {
                return dataSource.getChildren(dataItem);
            }
            var childrenField = this.get_containingField().get_childrenField();
            if (childrenField == null)
                throw Error.argumentNull('childrenField', 'The childrenField can not be null.');
            return dataItem[childrenField];
        },

        _showExpandButton: function() {
            Sys.UI.DomElement.setVisible(this._expandButton, true);
            this.get_contentContainer().style.marginLeft = '0px';
        },
        _hideExpandButton: function() {
            Sys.UI.DomElement.setVisible(this._expandButton, false);
            this.get_contentContainer().style.marginLeft = (this.get_depth() * 10 + 8) + 'px';
        },

        _createExpandButton: function() {
            var cell = this;
            var col = this.get_containingField();
            this._expandButton = document.createElement('span');
            this._expandButton.className = 'ui-icon';
            this._setButtonStyle();
            this._expandButton.style.marginLeft = (cell.get_depth() * 10) + 'px';

            var url = document.createElement('a');
            url.href = 'javascript:';
            url.appendChild(this._expandButton);
            this.get_element().appendChild(url);

            $addHandler(url, 'click', function(args) {
                cell.expand();
            });
        },

        initialize: function() {

            JData.TreeColumnCell.callBaseMethod(this, 'initialize');

            var row = this.get_parent();
            var col = this.get_containingField();

            var gridView = this.get_parent().get_parent();

            this.row = row;
            this.gridView = gridView;

            var cell = this;
            var fieldIndex = col.get_index();

            var children = this._getChildren(row.get_dataItem());
            this._createExpandButton();
            if (children != null && children.length > 0) {
                this._showExpandButton();
            }
            else {
                this._hideExpandButton();
            }
        }
    };

    JData.TreeColumnCell.registerClass('JData.TreeColumnCell', JData.EditableCell);

    //================================ End Internal Class =====================================
})();

JData.TreeColumn = function(field, headerText, itemWidth, controlWidth, readOnly) {
    /// <param name="field" type="String" mayBeNull="true">
    /// The name of the data field to bind to the BoundField object.
    /// </param>
    /// <param name="headerText" type="String" mayBeNull="true">
    /// The text that is displayed in the header item of a data control field
    /// </param>
    /// <param name="itemWidth" type="String" mayBeNull="true">
    /// The with of any cell generated by a data control field.
    /// </param>
    /// <param name="controlWidth" type="String" mayBeNull="true">
    /// The with of any control generated by a cell that generated by a data control field.
    /// </param>
    /// <param name="readOnly" type="Boolean" mayBeNull="true"/>

    JData.TreeColumn.initializeBase(this, [field, headerText, itemWidth, controlWidth, readOnly]);
};

JData.TreeColumn.prototype = {
    _set_owner: function(value) {
        JData.TreeColumn.callBaseMethod(this, '_set_owner', [value]);
        var col = this;
        var gridView = value;
        var treeColumn = this;

        var preParent;
        var _onDataSourceUpdated = function(sender1, args1) {
            args1.item = args1.row.get_dataItem();
            var dataSource = sender1.get_dataSource();
            
            var curParent;
            //preParent = dataSource.getParent(args1.oldItem);
            curParent = dataSource.getParent(args1.row.get_dataItem());

            var getCell = function(dataItem) {
                for (var i = 0; i < gridView.get_rows().length; i++) {
                    var row = gridView.get_rows()[i];
                    if (dataItem == row.get_dataItem()) {
                        cell = row.get_cells()[treeColumn.get_index()];
                        return cell;
                    }
                }
            }

            var getGridViewRow = function(index) {
                var j = 0;
                var rows = gridView.get_rows();
                for (var i = 0; i < gridView.get_rows().length; i++) {
                    if (rows[i].get_cells()[treeColumn.get_index()].get_parentCell() != null) {
                        if (j == index)
                            return rows[i];
                        else
                            j = j + 1;
                    }
                }
            }

            if (preParent != curParent) {
                var cell1 = getCell(curParent);
                var cell2 = getCell(preParent);
                var cell = getCell(args1.row.get_dataItem());
                Sys.Debug.assert(cell != null);

                if (cell2 != null) {
                    //cell2._removeChildRow(cell.get_parent());
                    cell2._removeChild(cell);
                }
                else {
                    //cell._removeChildRows();
                    cell._removeChildren();
                    gridView._RemoveRow(cell.get_parent());
                }

                if (cell1 != null) {
                    cell1._createChild(0, args1.item);
                }
                else {
                    var items = dataSource.getChildren(dataSource.get_root());
                    var index = Array.indexOf(items, args1.item);
                    var i = gridView.get_rows().length; //col._calculateIndex(null, index);
                    gridView._CreateDataRow(i, args1.item);
                }
            }
        };

        gridView.add_rowUpdating(function(sender, args) {
            preParent = gridView.get_dataSource().getParent(args.row.get_dataItem());
        });
        gridView.add_rowUpdated(_onDataSourceUpdated);

        gridView.add_rowRemoved(function(sender, args) {
            if (args.row.get_rowType() != JData.DataControlRowType.DataRow)
                return;

            var treeCell = args.row.get_cells()[col.get_index()];
            treeCell._removeChildren();
            var parentCell = treeCell.get_parentCell();
            if (parentCell == null)
                return;

            Array.remove(parentCell.get_childCells(), treeCell);
            if (parentCell.get_childCells().length == 0)
                parentCell._hideExpandButton();
        });

        gridView.add_rowDeleted(function(sender, args) {
            if (args.row.get_rowType() != JData.DataControlRowType.DataRow)
                return;

            var treeCell = args.row.get_cells()[col.get_index()];
            var parentCell = treeCell.get_parentCell();
            if (parentCell == null)
                return;
            if (parentCell.get_childCells().length < 1) {
                parentCell._hideExpandButton();
            }
        });

        gridView._onDataSourceInserted = function(sender, args) {
            if (args.item == null)
                throw Error.argumentNull('args.item');

            var dataSource = gridView.get_dataSource();
            Sys.Debug.assert(args.item != dataSource.get_root());

            for (var i = 0; i < gridView.get_rows().length; i++) {
                var row = gridView.get_rows()[i];
                if (row.get_dataItem() == args.item) {
                    return;
                }
            }
            var parentItem = dataSource.getParent(args.item);
            if (parentItem == dataSource.get_root() && !dataSource.get_displayRoot()) {
                gridView._CreateDataRow(gridView.get_rows().length, args.item);
            }
            else {
                for (var i = 0; i < gridView.get_rows().length; i++) {
                    var row = gridView.get_rows()[i]
                    if (row.get_dataItem() == parentItem) {
                        var treeColumnCell = row.get_cells()[col.get_index()];
                        var childCells = treeColumnCell.get_childCells();
                        if (childCells == null) {
                            treeColumnCell._showExpandButton();
                        }
                        else {
                            treeColumnCell._createChild(childCells.length, args.item);
                        }
                    }
                }
            }
        };
    },



    _getDataSource: function() {
        Sys.Debug.assert(this.gridView != null);
        var dataSource = this.gridView.get_dataSource();
        if (dataSource == null)
            throw Error.argumentNull('gridView.dataSource');

        if (JData.TreeDataSource.isInstanceOfType(dataSource) == false)
            throw Error.argumentType('gridView.dataSource', Object.getType(dataSource), JData.TreeDataSource);

        return dataSource;
    },

    get_cellType: function() {
        return JData.TreeColumnCell;
    },

    get_childrenField: function() {
        return this._childrenField;
    },
    set_childrenField: function(value) {
        this._childrenField = value;
    },
    get_parentField: function() {
        return this._parentField;
    },
    set_parentField: function(value) {
        this._parentField = value;
    },

    _getAllDisplayChildrenCount: function(cell) {
        var childCells = cell.get_childCells();
        if (!cell.get_expanded() || childCells == null) {
            return 0;
        }

        var count = childCells.length;
        for (var i = 0; i < childCells.length; i++) {
            count = count + this._getAllDisplayChildrenCount(childCells[i]);
        }
        return count;
    },

    initialize: function() {

    }
};

JData.TreeColumn.registerClass('JData.TreeColumn', JData.BoundField);

//========================== TreeParentColumn ==============================================

JData.TreeParentColumn = function(field, headerText, itemWidth, controlWidth) {
    /// <param name="field" type="String" mayBeNull="true">
    /// The name of the data field to bind to the BoundField object.
    /// </param>
    /// <param name="headerText" type="String" mayBeNull="true">
    /// The text that is displayed in the header item of a data control field
    /// </param>
    /// <param name="itemWidth" type="String" mayBeNull="true">
    /// The with of any cell generated by a data control field.
    /// </param>
    /// <param name="controlWidth" type="String" mayBeNull="true">
    /// The with of any control generated by a cell that generated by a data control field.
    /// </param>

    this._displayRoot = true;
    JData.TreeParentColumn.initializeBase(this, [field, headerText, itemWidth, controlWidth]);
}

JData.TreeParentColumn.prototype = {
    _set_owner: function(value) {
        JData.TreeColumn.callBaseMethod(this, '_set_owner', [value]);
        this.gridView = value;
    },

    _GetDataItems: function(cell, successedCallback, failedCallback) {
        var dataSource = this.gridView.get_dataSource();

        if (dataSource == null)
            throw Error.argumentNull("dataSource");

        if (JData.TreeDataSource.isInstanceOfType(dataSource) == false)
            throw Error.argumentType('dataSource', Object.getType(dataSource), JData.TreeDataSource);

        var result = new Array();
        var rowDataItem = cell.get_parent().get_dataItem();
        var appendItem = function(item) {
            if (item == rowDataItem) {
                return;
            }
            Array.add(result, item);
            var children = dataSource.getChildren(item);
            for (var i = 0; i < children.length; i++) {
                item = children[i];
                appendItem(item);
            }
        }
        appendItem(dataSource.get_root());
        successedCallback(result);
    },

    _CreateCell: function(gridRow) {
        var cell = JData.TreeParentColumn.callBaseMethod(this, '_CreateCell', new Array(gridRow));
        cell._getDisplay = function(item) {
            var displayMember = this.get_containingField().get_displayMember();
            Sys.Debug.assert(displayMember != null);
            var dataSource = cell.get_parent().get_parent().get_dataSource();

            var parent;
            var display
            if (item != null)
                display = item[displayMember];
            else
                display = this.get_containingField().get_nullText();

            if (item == dataSource.get_root()) {
            }
            else {
                parent = dataSource.getParent(item);
                while (parent != null && parent != dataSource.get_root()) {
                    display = "&nbsp;&nbsp;&nbsp;&nbsp;" + display;
                    parent = dataSource.getParent(parent);
                }
            }
            return display;
        };

        return cell;
    }
}

JData.TreeParentColumn.registerClass('JData.TreeParentColumn', JData.DropDownListField);

//============================= TreeDataSource =================================

JData.TreeDataSource = function(url, serviceSelectMethodName, serviceInsertMethodName,
                                     serviceUpdateMethodName, serviceDeleteMethodName, selector, sorting, filter) {

    JData.TreeDataSource.initializeBase(this, [url, serviceSelectMethodName, serviceInsertMethodName,
                                                    serviceUpdateMethodName, serviceDeleteMethodName, selector, sorting, filter]);
    this.set_enableCache(true);
    var dataSource = this;
    dataSource.add_inserted(function(sender, args) {
        Array.add(dataSource._items(), args.item);
    });
    dataSource.add_deleted(function(sender, args) {
        Array.remove(dataSource._items(), args.item);
    });
};

JData.TreeDataSource.prototype = {

    get_root: function() {
        if (this.get_rootID() == null)
            return null;

        var root = null;
        var items = this._items();
        for (var i = 0; i < items.length; i++) {
            if (items[i][dataSource.get_primaryKeyColumn()] == this.get_rootID()) {
                root = items[i];
                break;
            }
        }
        return root;
    },

    get_displayRoot: function() {
        if (this.get_root() == null)
            return false;

        return this._displayRoot;
    },
    set_displayRoot: function(value) {
        this._displayRoot = value;
    },

    getChildrenCount: function(dataItem) {
        return this.getChildren(dataItem).length;
    },

    get_rootID: function() {
        return this._rootID;
    },
    set_rootID: function(value) {
        this._rootID = value;
    },

    get_primaryKeyColumn: function() {
        /// <return type="String"/>
        return this._primaryKeyColumn;
    },
    set_primaryKeyColumn: function(value) {
        /// <param name="value" type="String"/>
        this._primaryKeyColumn = value;
    },

    get_parentColumn: function() {
        /// <return type="String"/>
        return this._parentColumn;
    },
    set_parentColumn: function(value) {
        /// <param name="value" type="String"/>
        this._parentColumn = value;
    },

    _items: function() {
        /// <returns type="Array"/>
        if (this.$items != null)
            return this.$items;

        var postData = {};
        postData.skip = -1;
        postData.take = -1;
        postData.totalRowCount = -1;
        postData.retrieveTotalRowCount = false;
        postData.sorting = this.get_sorting();
        postData.selector = this.get_selector();
        postData.filter = this.get_filter();

        var dataSource = this;
        var result = $.ajax({
            async: false,
            url: this.get_url() + '/' + this.get_selectMethodName(),
            type: "POST",
            contentType: "application/json;utf-8",
            data: Sys.Serialization.JavaScriptSerializer.serialize(postData)
        });
        if (result.status == 200) {
            var json = Sys.Serialization.JavaScriptSerializer.deserialize(result.responseText);
            this.$items = json['d']['Items'];
            return this.$items;
        }
        alert(result.responseText);
    },

    getChildren: function(dataItem) {
        var items = this._items();
        var arr = new Array();
        var parentID;
        if (dataItem == this.get_root())
            parentID = this.get_rootID();
        else
            parentID = dataItem[this.get_primaryKeyColumn()];

        for (var i = 0; i < items.length; i++) {
            if (items[i][this.get_parentColumn()] == parentID) {
                Sys.Debug.assert(items[i] != null);
                Array.add(arr, items[i]);
            }
        }
        return arr;

    },

    getParent: function(dataItem) {
        Sys.Debug.assert(this.get_primaryKeyColumn() != null);
        Sys.Debug.assert(this.get_parentColumn() != null);

        var items = this._items();
        var parent = null;
        for (var i = 0; i < items.length; i++) {
            if (items[i][this.get_primaryKeyColumn()] == dataItem[this.get_parentColumn()]) {
                parent = items[i];
                break;
            }
        }
        return parent;
    },

    executeSelect: function(args, successedCallback, failedCallback) {
        if (this.get_displayRoot() == true) {
            var array = new Array();
            Sys.Debug.assert(this.get_root() != null && this.get_root() != undefined);
            Array.add(array, this.get_root());
            successedCallback(array);
        }
        else
            successedCallback(this.getChildren(this.get_root()));
    }
};

JData.TreeDataSource.registerClass('JData.TreeDataSource', JData.WebServiceDataSource);



