﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Services;
using JData.Services;

namespace WebApplication
{
    public class NorthwindService : NorthwindServiceBase
    {
        [WebMethod]
        public ExecuteResult UpdateOrderWithValidate(Dictionary<string, object> item)
        {

            string freightError = null;
            string shipNameError = null;

            var Freight = Convert.ToInt32(item["Freight"]);
            var ShipName = Convert.ToString(item["ShipName"]);

            //Validate the values.
            if (Freight > 2)
                freightError = "Please enter a value less than or equal to 2.";

            if (ShipName == null || ShipName.Length > 5)
                shipNameError = "Please enter no more than 5 characters.";

            if (freightError != null || shipNameError != null)
            {
                var result = new { Freight = freightError, ShipName = shipNameError };
                return new ExecuteResult { AffectedRowCount = 0, Message = result };
            }

            //Update the data
            return new ExecuteResult { AffectedRowCount = 1 };
        }

        [WebMethod]
        public override ExecuteResult InsertOrder(Dictionary<string, object> item)
        {
            var db = base.CreateDataContext();
            item["OrderID"] = db.Orders.Max(o => o.OrderID) + 1;
            var result = base.InsertOrder(item);
            result.Members = new { OrderID = item["OrderID"] };
            return result;
        }

        class DataItem
        {
            public int EmployeeID, JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC;
            public string EmployeeName;
            public string ParentName;
        }

        [WebMethod]
        public QueryResult GetOrderReport()
        {
            var MONTHS = new[] { "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };

            var db = new NorthwindDatabase();
            var items = db.Orders.Where(o => o.OrderDate != null).GroupBy(o => new { o.EmployeeID, o.OrderDate.Value.Month })
                                 .Select(o => new { o.Key.EmployeeID, o.Key.Month, Count = o.Count() }).ToArray();
            var list = new List<DataItem>();
            foreach (var employee in db.Employees.Select(o => new { o.EmployeeID, o.FirstName, o.LastName }))
            {
                var dataItem = new DataItem { EmployeeID = employee.EmployeeID, EmployeeName = employee.FirstName + " " + employee.LastName };

                for (var i = 0; i < MONTHS.Length; i++)
                {
                    var count = 0;
                    var item = items.Where(o => o.EmployeeID == dataItem.EmployeeID && o.Month == i + 1).SingleOrDefault();
                    if (item != null)
                        count = item.Count;
                    var field = typeof(DataItem).GetField(MONTHS[i]);
                    field.SetValue(dataItem, count);
                }
                list.Add(dataItem);
            }
            var resut = new QueryResult() { Items = list.ToArray() };
            return resut;
        }

        [WebMethod]
        public QueryResult GetEmployeeReport()
        {
            var db = new NorthwindDatabase();
            var q = db.Employees.OrderBy(o => o.ReportsTo)
                                .Select(o => new
                                {
                                    ResponseTo = o.Employee1 == null ? " " : string.Format("{0} {1}", o.Employee1.FirstName, o.Employee1.LastName),
                                    o.EmployeeID,
                                    Name = o.FirstName + " " + o.LastName,
                                    o.HireDate,
                                    o.Region,
                                    o.Country,
                                    o.City
                                });
            var result = new QueryResult { Items = q.ToArray() };
            return result;
        }

        public override ExecuteResult DeleteOrder(Dictionary<string, object> item)
        {
#if REMOTE
            if ((int)item["OrderID"] <= 11077)
            {
                var result = new ExecuteResult { AffectedRowCount = 0, Message = "The record which orderID <= 11077  not allow deleted for demo." };
                return result;
            }
#endif
            return base.DeleteOrder(item);
        }

        public override ExecuteResult DeleteEmployee(Dictionary<string, object> item)
        {
#if REMOTE
            if ((int)item["EmployeeID"] <= 9)
            {
                var result = new ExecuteResult { AffectedRowCount = 0, Message = "The record which employeeID <= 9  not allow deleted for demo." };
                return result;
            }
#endif
            return base.DeleteEmployee(item);
        }
    }
}
